package notpowerpoint;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;

import javax.swing.*;

/**
 * Represents the menu bar for a SlidePanel, meant to be
 * used in a window where the content pane is a SlidePanel.
 * An object of type Menus has an "owner" which is of type
 * SlidePanel.  Menu commands are carried out by calling
 * methods in the SlidePanel.
 */
public class Menus extends JMenuBar {
	

	private final SlidePanel owner;  // This menu bar's owner.
	
	
	/**
	 * Create a menu bar with a given owner of type SlidePanel.
	 */
	public Menus(SlidePanel owner) {
		this.owner = owner;
		add( createFileMenu() );
		add( createBackgroundMenu() );
	}
	
	
	/**
	 * Add a keyboard accelerator to a menu item, such as a "CONTROL-Q" for
	 * the quit command.  The Mac command key is used in Mac OS, and the 
	 * control key is used otherwise.
	 * @param item  The menu item to which the accelerator will be applied.
	 * @param key  The character that will be pressed, in addition to the
	 *    control key, to trigger the menu command.  Generally, an UPPER CASE
	 *    letter.
	 */
	private void addAccelerator(JMenuItem item, char key) {
		String stroke;
		if (System.getProperty("mrj.version") == null)
			stroke = "control " + key; // for non-Mac computers
		else
			stroke = "meta " + key; // for Mac OS
		item.setAccelerator(KeyStroke.getKeyStroke(stroke));
	}
	
	
	/**
	 *  Create the background menu, which allows the user to select the
	 *  background for the current slide and the background color for
	 *  the text area of the current slide.
	 */
	private JMenu createFileMenu() {
		ActionListener listener = new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				String cmd = evt.getActionCommand();
				if (cmd.equals("Quit"))
					owner.doClose();
			}
		};
		JMenu fileMenu = new JMenu("File");
		JMenuItem quit= new JMenuItem("Quit");
		quit.addActionListener(listener);
		addAccelerator(quit, 'Q');
		fileMenu.add(quit);
		return fileMenu;
	}
	
	private JMenu createBackgroundMenu() {
		final Color[] colors = { Color.WHITE, Color.BLACK, Color.GRAY, Color.LIGHT_GRAY, Color.BLUE, 
				new Color(200,200,255), Color.RED, Color.PINK, new Color(255,230,180) };
		String[] colorNames = { "White", "Black", "Gray", "Light Gray", "Blue", 
				"Light Blue", "Red", "Pink", "Beige" };
		ActionListener listener = new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				String cmd = evt.getActionCommand();
				if (cmd.equals("None")) {
					owner.setTextAreaColor(null);
				}
				else if (cmd.equals("Gradient Background...")) {
					doGradientBackground();
				}
				else if (cmd.equals("CustomBG")) {
					Color c = JColorChooser.showDialog(owner, "Select Background Color", Color.WHITE);
					if (c != null)
						owner.setColorBackground(c);
				}
				else if (cmd.equals("CustomTAC")) {
					Color c = JColorChooser.showDialog(owner, "Select Text Area Background Color", Color.WHITE);
					if (c != null)
						owner.setTextAreaColor(c);
				}
				else if (cmd.startsWith("tac")) {
					int x = Integer.parseInt(cmd.substring(3));
					owner.setTextAreaColor(colors[x]);
				}
				else if (cmd.startsWith("cb")) {
					int x = Integer.parseInt(cmd.substring(2));
					owner.setColorBackground(colors[x]);
				}
				else if (cmd.startsWith("tex")) {
					int x = Integer.parseInt(cmd.substring(3));
					owner.setTextureBackground(x);
				}
			}
		};
		JMenu menu = new JMenu("Background");
		JMenu textureBackground = new JMenu("Texture Background");
		JMenu colorBackground = new JMenu("Color Background");
		JMenu textAreaColor = new JMenu("Text Area Color");
		JMenuItem gradientBackground = new JMenuItem("Gradient Background...");
		gradientBackground.addActionListener(listener);
		menu.add(colorBackground);
		menu.add(textureBackground);
		menu.add(gradientBackground);
		menu.addSeparator();
		menu.add(textAreaColor);
		for (int i = 0; i < colorNames.length; i++) {
			JMenuItem item = new JMenuItem(colorNames[i]);
			item.addActionListener(listener);
			item.setActionCommand("cb" + i);
			colorBackground.add(item);
		}
		colorBackground.addSeparator();
		JMenuItem customBC = new JMenuItem("Custom...");
		customBC.setActionCommand("CustomBG");
		customBC.addActionListener(listener);
		colorBackground.add(customBC);
	    JMenuItem noColor = new JMenuItem("None");
	    noColor.addActionListener(listener);
	    textAreaColor.add(noColor);
	    textAreaColor.addSeparator();
		for (int i = 0; i < colorNames.length; i++) {
			JMenuItem item = new JMenuItem(colorNames[i]);
			item.addActionListener(listener);
			item.setActionCommand("tac" + i);
			textAreaColor.add(item);
		}
		textAreaColor.addSeparator();
		JMenuItem customTAC = new JMenuItem("Custom...");
		customTAC.setActionCommand("CustomTAC");
		customTAC.addActionListener(listener);
		textAreaColor.add(customTAC);
		for (int i = 0; i < SlideInfo.getTextureImageCount(); i++) {
			BufferedImage img = SlideInfo.getTextureImage(i);
			BufferedImage icon = new BufferedImage(48,48,BufferedImage.TYPE_INT_ARGB);
			Graphics g = icon.getGraphics();
			g.drawImage(img,0,0,48,48,null);
			g.dispose();
			JMenuItem item = new JMenuItem( new ImageIcon(icon) );
			item.setActionCommand("tex" + i);
			item.addActionListener(listener);
			textureBackground.add(item);
		}
		return menu;
	}
	
	
	/**
	 * This method is called when the user selects a Gradient background
	 * from the background menu.  It lets the user select the orientation
	 * and colors of the gradient.  All gradients cover the entire extent
	 * of the slide, with either a horizontal, vertical, or diagonal
	 * orientation.
	 */
	private void doGradientBackground() {
		MouseListener ml = new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				JPanel src = (JPanel)evt.getSource();
				Color c = src.getBackground();
				c = JColorChooser.showDialog(src, "Select Gradient Color", c);
				if (c != null)
					src.setBackground(c);
			}
		};
		JPanel colorPatch1 = new JPanel();
		colorPatch1.setBackground(Color.DARK_GRAY);
		colorPatch1.setPreferredSize( new Dimension(80,40) );
		colorPatch1.setBorder(BorderFactory.createLineBorder(Color.BLACK,2));
		colorPatch1.addMouseListener(ml);
		JPanel colorPatch2 = new JPanel();
		colorPatch2.setBackground(Color.LIGHT_GRAY);
		colorPatch2.setPreferredSize( new Dimension(80,40) );
		colorPatch2.addMouseListener(ml);
		colorPatch2.setBorder(BorderFactory.createLineBorder(Color.BLACK,2));
		JRadioButton hor = new JRadioButton("Horizontal");
		JRadioButton vert = new JRadioButton("Vertical");
		JRadioButton asc = new JRadioButton("Ascending Diagonal");
		JRadioButton desc = new JRadioButton("Descending Diagonal");
		ButtonGroup group = new ButtonGroup();
		group.add(hor);
		group.add(vert);
		group.add(asc);
		group.add(desc);
		desc.setSelected(true);
		JPanel top = new JPanel();
		top.setLayout( new GridLayout(3,2,8,8) );
		top.add(new JLabel("Orientation:"));
		top.add(new JLabel()); // placeholder
		top.add(hor);
		top.add(vert);
		top.add(asc);
		top.add(desc);
		JPanel bottom = new JPanel();
		bottom.setLayout(new GridLayout(2,2,8,8));
		bottom.add(new JLabel("Colors:"));
		bottom.add(new JLabel("(click to change)"));
		bottom.add(colorPatch1);
		bottom.add(colorPatch2);
		JPanel input = new JPanel();
		input.setLayout(new BorderLayout(15,15));
		input.add(top,BorderLayout.NORTH);
		input.add(bottom,BorderLayout.CENTER);
		input.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		int resp = JOptionPane.showConfirmDialog(owner, input, "Select Gradient Options", JOptionPane.OK_CANCEL_OPTION);
		if (resp != JOptionPane.OK_OPTION)
			return;
		Color c1 = colorPatch1.getBackground();
		Color c2 = colorPatch2.getBackground();
		int x1 = 0, y1 = 0, x2 = owner.getWidth(), y2 = owner.getHeight();
		if (hor.isSelected())
			y2 = 0;
		else if (vert.isSelected()) 
			x2 = 0;
		else if (asc.isSelected()) {
			y1 = owner.getHeight();
			y2 = 0;
		}
		owner.setGradientBackground(x1, y1, c1, x2, y2, c2);
	}

}
