package notpowerpoint;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.net.URL;

import javax.imageio.ImageIO;

/**
 * Holds a complete description of one slide, including
 * background and content.
 */
public class SlideInfo {
	
	private static final String[] textureImageResourceNames = { // paths to texture images
		"notpowerpoint/textures/beige_paper.jpg",
		"notpowerpoint/textures/blue_paper.jpg",
		"notpowerpoint/textures/bolts.gif",
		"notpowerpoint/textures/brushed-metal.jpg",
		"notpowerpoint/textures/celtic-knot.jpg",
		"notpowerpoint/textures/cinderblock.jpg",
		"notpowerpoint/textures/gray_mosaic.jpg",
		"notpowerpoint/textures/marble.jpg",
		"notpowerpoint/textures/pink_marble.gif",
		"notpowerpoint/textures/pink_roses.jpg",
		"notpowerpoint/textures/sky.jpg",
		"notpowerpoint/textures/wood_floor.jpg"
	};
	
	private static final BufferedImage[] textureImages // texture images; null if not yet loaded
	            = new BufferedImage[textureImageResourceNames.length];
	
	/**
	 * Constants representing the possible types of background
	 * used on the slide.
	 */
	public static final int
	    COLOR_BACKGROUND = 1,
	    GRADIENT_BACKGROUND = 2,
	    TEXTURE_BACKGROUND = 3;
	
	/**
	 * What type if background is used.  The value is one
	 * of the constants COLOR_BACKGROUND, GRADIENT_BACKGROUND,
	 * or TEXTURE_BACKGROUND.
	 */
	public int backgroundStyle = COLOR_BACKGROUND;
	
	
	/**
	 * The background color, IF backgroundStyle is COLOR_BACKGROUND.
	 */
	public Color backgroundColor = Color.WHITE;
	
	
	/**
	 * The index of the background texture, If the backgroundStyle
	 * is TEXTURE_BACKGROUND.  (This int is an index into an array
	 * of available textures.)
	 */
	public int textureNumber;
	
	
	/**
	 * The start and end colors for the gradient, IF the backgroundStyle
	 * is GRADIENT_BACKGROUND.
	 */
	public Color gradientColor1, gradientColor2;
	
	
	/**
	 * The start and end points for the gradient, IF the backgroundStyle
	 * is GRADIENT_BACKGROUND;
	 */
	public int gradientX1, gradientY1, gradientX2, gradientY2;
	
	
	/**
	 * The background color for the text area of the slide.  If the
	 * value is null, then the general background of the slide is the
	 * background of the text area as well.  If the value is not null,
	 * then the background of the slide just occupies a border around
	 * the text area.  
	 */
	public Color textAreaBackgroundColor = null;
	
	
	/**
	 * Returns the number of texture images.  Images are numbered
	 * from 0 to this value minus 1.
	 */
	public static int getTextureImageCount() {
		return textureImages.length;
	}
	
	
	/**
	 * Returns the specified textured image.  A null return value indicates
	 * that the resource could not be loaded.
	 * @param imageNumber  The number of the image that is to be returned
	 *   Must be in the range 0 to getTextureImageCount() - 1.  A number
	 *   outside this range will result in a ArrayIndexOutOfBoundsException.
	 */
	public static BufferedImage getTextureImage(int imageNumber) {
		if (textureImages[imageNumber] == null) { // load texture if not already loaded
			try {
				ClassLoader cl = SlideInfo.class.getClassLoader();
				URL resourceLocation = cl.getResource(textureImageResourceNames[imageNumber]);
				BufferedImage img = ImageIO.read(resourceLocation);
				textureImages[imageNumber] = img;
			}
			catch (Exception e) {
			}
		}
		return textureImages[imageNumber];
	}

}
