package notpowerpoint;

import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import javax.swing.*;

public class SlidePanel extends JPanel {
	
	/**
	 * A class representing a panel that can display "slides".  A slide has
	 * a background, which can be a color, a texture, or a gradient.  Inside
	 * the slide is a large text area that can contain text.  The slide 
	 * background always shows in a 30-pixel border area around the text
	 * area.  It can also show behind the text, but the text area can also,
	 * optionally, have its own background color.
	 */
	private class SlideDisplay extends JPanel {
		SlideDisplay() {
			setPreferredSize( new Dimension(800,600) );
		}
		protected void paintComponent(Graphics g) {
			super.paintComponent(g);
			Graphics2D g2 = (Graphics2D)g;
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			if (currentSlide == null)
				return;
			if (currentSlide.backgroundStyle == SlideInfo.COLOR_BACKGROUND)
				g2.setPaint( currentSlide.backgroundColor );
			else if (currentSlide.backgroundStyle == SlideInfo.GRADIENT_BACKGROUND)
				g2.setPaint( new GradientPaint(currentSlide.gradientX1, currentSlide.gradientY1, currentSlide.gradientColor1,
						currentSlide.gradientX2, currentSlide.gradientY2, currentSlide.gradientColor2) );
			else {
				BufferedImage texture = SlideInfo.getTextureImage(currentSlide.textureNumber);
				g2.setPaint( new TexturePaint(texture,  
						new Rectangle2D.Double(0,0,texture.getWidth(),texture.getHeight())));
			}
			g2.fillRect(0,0,getWidth(),getHeight());
			if (currentSlide.textAreaBackgroundColor != null) {
				g2.setColor( currentSlide.textAreaBackgroundColor );
				g2.fillRoundRect(30, 30, getWidth()-60, getHeight()-60, 16, 16);
				g2.setColor(Color.BLACK);
				g2.setStroke( new BasicStroke(2) );
				g2.drawRoundRect(30, 30, getWidth()-60, getHeight()-60, 16, 16);
			}
		}
	}
	
	
	private SlideInfo currentSlide;  // The slide that is currently shown in the display panel.
	
	private SlideDisplay display;  // The display panel, which occupies most of this panel, except
	                               // for the message at the bottom.  Defined by a nested class.
	
	private JLabel message;  // Holds the message that appears at the bottom of the window.
	
	
	/**
	 * Create a SlidePanel object, with a preferred size of
	 * 800-by-600.  Also creates the current slide.
	 */
	public SlidePanel() {
		setLayout(new BorderLayout(3,3));
		setBackground(Color.GRAY);
		setBorder(BorderFactory.createLineBorder(Color.GRAY,2));
		currentSlide = new SlideInfo();
		display = new SlideDisplay();
		add(display,BorderLayout.CENTER);
		message = new JLabel("Slide Number 1");
		message.setBackground(Color.LIGHT_GRAY);
		message.setOpaque(true);
		message.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		add(message,BorderLayout.SOUTH);
	}

	
	/**
	 * This method is called when the user clicks the close box of
	 * the window and when the user chooses "Quit" from the "File"
	 * menu.  It ends the program (but gives the panel a chance to
	 * do something else, like asking the user if they really 
	 * want to quit).
	 */
	public void doClose() {
		System.exit(0);
	}
	
	
	/**
	 * Set the background color of the text area in the current slide.
	 * A null value means that there is no text area background color.
	 * In that case, the general background of the slide is also the
	 * background for the text.  If the color is non-null then a rectangle
	 * of solid color fills the text area of the slide, and a black outline
	 * is drawn around the area.  The default is to have no text area
	 * background color.
	 */
	public void setTextAreaColor(Color c) {
		currentSlide.textAreaBackgroundColor = c;
		display.repaint();
	}
	
	
	/**
	 * Set the current slide to use a texture image as a background.
	 * @param textureNumber The texture number, an integer between 0
	 *    and SlideInfo.getTextureImageCount() giving the index of
	 *    the texture in the list of available textures.
	 */
	public void setColorBackground(Color c) {
		if (c == null)
			c = Color.WHITE;
		currentSlide.backgroundStyle = SlideInfo.COLOR_BACKGROUND;
		currentSlide.backgroundColor = c;
		display.repaint();
	}
	

	/**
	 * Set the current slide to use a texture image as a background.
	 * @param textureNumber The texture number, an integer between 0
	 *    and SlideInfo.getTextureImageCount() giving the index of
	 *    the texture in the list of available textures.
	 */
	public void setTextureBackground(int textureNumber) {
		if (textureNumber < 0 || textureNumber >= SlideInfo.getTextureImageCount())
			textureNumber = 0;
		currentSlide.backgroundStyle = SlideInfo.TEXTURE_BACKGROUND;
		currentSlide.textureNumber = textureNumber;
		display.repaint();
	}
	
	
	/**
	 *  Set the current slide to use a gradient background, with the gradient changing
	 *  color c1 to color c2 between the points (x1,y1) and (x2,y2).
	 */
	public void setGradientBackground(int x1, int y1, Color c1, int x2, int y2, Color c2) {
		if (c1 == null)
			c1 = Color.LIGHT_GRAY;
		if (c2 == null)
			c2 = Color.GRAY;
		currentSlide.backgroundStyle = SlideInfo.GRADIENT_BACKGROUND;
		currentSlide.gradientX1 = x1;
		currentSlide.gradientY1 = y1;
		currentSlide.gradientColor1 = c1;
		currentSlide.gradientX2 = x2;
		currentSlide.gradientY2 = y2;
		currentSlide.gradientColor2 = c2;
		display.repaint();
	}
	

}
